@@ -1207,7 +1207,7 @@ gboolean project_load(appdata_t *appdata, char *name) {
   /* load diff possibly preset */
   banner_busy_tick();
   diff_restore(appdata, appdata->project, appdata->osm);
-  
+
   /* prepare colors etc, draw data and adjust scroll/zoom settings */
   banner_busy_tick();
   map_init(appdata);
@@ -1227,13 +1227,298 @@ gboolean project_load(appdata_t *appdata, char *name) {
     g_free(appdata->settings->project);
   appdata->settings->project = g_strdup(appdata->project->name);
 
-  snprintf(banner_txt, _PROJECT_LOAD_BUF_SIZ, _("Loaded %s"), proj_name);
-
   banner_busy_stop(appdata);
+
+#if 0
+  snprintf(banner_txt, _PROJECT_LOAD_BUF_SIZ, _("Loaded %s"), proj_name);
   banner_show_info(appdata, banner_txt);
+#endif
+
   statusbar_set(appdata, NULL, 0);
 
   g_free(proj_name);
   return TRUE;
 }
+
+/* ------------------- project setup wizard ----------------- */
+
+struct wizard_s;
+
+typedef struct wizard_page_s {
+  const gchar *title;
+  GtkWidget* (*setup)(struct wizard_page_s *page);
+  void (*update)(struct wizard_page_s *page);
+  GtkAssistantPageType type;
+  gboolean complete;
+  /* everything before here is initialized statically */
+
+  struct wizard_s *wizard;
+  GtkWidget *widget;
+  gint index;
+
+  union {
+    struct {
+      GtkWidget *check[3];
+      GtkWidget *label[3];
+    } source_selection;
+
+  } state;
+
+} wizard_page_t;
+
+typedef struct wizard_s {
+  gboolean running;
+
+  int page_num;
+  wizard_page_t *page;
+  appdata_t *appdata;
+  guint handler_id;
+  GtkWidget *assistant;
+} wizard_t;
+
+
+static gint on_assistant_destroy(GtkWidget *widget, wizard_t *wizard) {
+  printf("destroy callback\n");
+  wizard->running = FALSE;
+  return FALSE;
+}
+
+static void on_assistant_cancel(GtkWidget *widget, wizard_t *wizard) {
+  printf("cancel callback\n");
+  wizard->running = FALSE;
+}
+
+static void on_assistant_close(GtkWidget *widget, wizard_t *wizard) {
+  printf("close callback\n");
+  wizard->running = FALSE;
+}
+
+static GtkWidget *wizard_text(const char *text) {
+  GtkTextBuffer *buffer = gtk_text_buffer_new(NULL);
+  gtk_text_buffer_set_text(buffer, text, -1);
+
+#ifndef USE_HILDON_TEXT_VIEW
+  GtkWidget *view = gtk_text_view_new_with_buffer(buffer);
+#else
+  GtkWidget *view = hildon_text_view_new();
+  hildon_text_view_set_buffer(HILDON_TEXT_VIEW(view), buffer);
+#endif
+
+  gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(view), GTK_WRAP_WORD);
+  gtk_text_view_set_editable(GTK_TEXT_VIEW(view), FALSE);
+  gtk_text_view_set_left_margin(GTK_TEXT_VIEW(view), 2 );
+  gtk_text_view_set_right_margin(GTK_TEXT_VIEW(view), 2 );
+  gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(view), FALSE );
+
+  return view;
+}
+
+/* ---------------- page 1: intro ----------------- */
+static GtkWidget *wizard_create_intro_page(wizard_page_t *page) {
+  static const char *text = 
+    "This wizard will guide you through the setup of a new project.\n\n"
+    "An osm2go project covers a certain area of the world as seen "
+    "by openstreetmap.org. The wizard will help you downloading "
+    "the data describing that area and will enable you to make changes "
+    "to it using osm2go.";
+
+  return wizard_text(text);
+}
+
+/* ---------------- page 2: source selection ----------------- */
+static gboolean gtk_widget_get_sensitive(GtkWidget *widget) {
+  GValue is_sensitive= { 0, };
+  g_value_init(&is_sensitive, G_TYPE_BOOLEAN);
+  g_object_get_property(G_OBJECT(widget), "sensitive", &is_sensitive);
+  return g_value_get_boolean(&is_sensitive);
+}
+
+static void wizard_update_source_selection_page(wizard_page_t *page) {
+
+  gboolean gps_on = page->wizard->appdata && 
+    page->wizard->appdata->settings && 
+    page->wizard->appdata->settings->enable_gps;
+  gboolean gps_fix = gps_on && gps_get_pos(page->wizard->appdata, NULL, NULL);
+
+  gtk_widget_set_sensitive(page->state.source_selection.check[0], gps_fix);
+  if(gps_fix) 
+    gtk_label_set_text(GTK_LABEL(page->state.source_selection.label[0]),
+		       "(GPS has a valid position)");
+  else if(gps_on)
+    gtk_label_set_text(GTK_LABEL(page->state.source_selection.label[0]),
+		       "(GPS has no valid position)");
+  else
+    gtk_label_set_text(GTK_LABEL(page->state.source_selection.label[0]),
+		       "(GPS is disabled)");
+
+#ifndef USE_HILDON
+  gtk_widget_set_sensitive(page->state.source_selection.check[1], FALSE);
+  gtk_label_set_text(GTK_LABEL(page->state.source_selection.label[1]),
+		       "(Maemo Mapper not available)");
+
+#endif
+
+  /* check if the user selected something that is actually selectable */
+  /* only allow him to continue then */
+  gboolean sel_ok = FALSE;
+  int i;
+  for(i=0;i<3;i++) {
+    if(gtk_toggle_button_get_active(
+	    GTK_TOGGLE_BUTTON(page->state.source_selection.check[i])))
+      sel_ok = gtk_widget_get_sensitive(page->state.source_selection.check[i]);
+  }
+
+  /* set page to "completed" if a valid entry is selected */
+  gtk_assistant_set_page_complete(
+               GTK_ASSISTANT(page->wizard->assistant), page->widget, sel_ok);
+}
+
+/* the user has changed the selected source, update dialog */
+static void on_wizard_source_selection_toggled(GtkToggleButton *togglebutton,
+					    gpointer user_data) {
+  if(gtk_toggle_button_get_active(togglebutton)) 
+    wizard_update_source_selection_page((wizard_page_t*)user_data);
+}
+
+static GtkWidget *wizard_create_source_selection_page(wizard_page_t *page) {
+  GtkWidget *vbox = gtk_vbox_new(FALSE, 0);
+
+  gtk_box_pack_start_defaults(GTK_BOX(vbox), 
+	      wizard_text("Please choose how to determine the area you "
+			  "are planning to work on."));
+
+  GtkWidget *hbox = gtk_hbox_new(FALSE, 0);
+  GtkWidget *vbox2 = gtk_vbox_new(FALSE, 0);
+
+  /* add selection buttons */
+  int i;
+  for(i=0;i<3;i++) {
+    static const char *labels[] = {
+      "Use current GPS position",
+      "Get from Maemo Mapper",
+      "Specify area manually"
+    };
+
+    page->state.source_selection.check[i] = 
+      gtk_radio_button_new_with_label_from_widget(
+	      i?GTK_RADIO_BUTTON(page->state.source_selection.check[0]):NULL, 
+	      _(labels[i]));
+    g_signal_connect(G_OBJECT(page->state.source_selection.check[i]), 
+	"toggled", G_CALLBACK(on_wizard_source_selection_toggled), page);
+    gtk_box_pack_start(GTK_BOX(vbox2), page->state.source_selection.check[i], 
+		       TRUE, TRUE, 2);
+    page->state.source_selection.label[i] = gtk_label_new("");
+    gtk_box_pack_start(GTK_BOX(vbox2), page->state.source_selection.label[i], 
+		       TRUE, TRUE, 2);
+  }
+
+  gtk_box_pack_start(GTK_BOX(hbox), vbox2, TRUE, FALSE, 0); 
+  gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0); 
+  return vbox;
+}
+
+/* this is called once a second while the wizard is running and can be used */
+/* to update pages etc */
+static gboolean wizard_update(gpointer data) {
+  wizard_t *wizard = (wizard_t*)data;
+  gint page = gtk_assistant_get_current_page(GTK_ASSISTANT(wizard->assistant));
+
+  if(wizard->page[page].update)
+ ;   //    wizard->page[page].update(&wizard->page[page]);
+  else
+    printf("nothing to animate on page %d\n", page);
+
+  return TRUE;
+}
+
+void project_wizard(appdata_t *appdata) {
+  wizard_page_t page[] = {
+    { "Introduction",           wizard_create_intro_page, NULL,
+      GTK_ASSISTANT_PAGE_INTRO,    TRUE},
+    { "Area source selection",  wizard_create_source_selection_page,
+      wizard_update_source_selection_page,
+      GTK_ASSISTANT_PAGE_CONTENT,  FALSE},
+    { "Click the Check Button", NULL, NULL,
+      GTK_ASSISTANT_PAGE_CONTENT,  FALSE},
+    { "Click the Button",       NULL, NULL,
+      GTK_ASSISTANT_PAGE_PROGRESS, FALSE},
+    { "Confirmation",           NULL, NULL,
+      GTK_ASSISTANT_PAGE_CONFIRM,  TRUE},
+  };
+  
+  wizard_t wizard = {
+    TRUE,
+
+    /* the pages themselves */
+    sizeof(page) / sizeof(wizard_page_t), page,
+    appdata, 0, NULL
+  };
+
+  wizard.assistant = gtk_assistant_new();
+  gtk_widget_set_size_request(wizard.assistant, 450, 300);
+
+  /* Add five pages to the GtkAssistant dialog. */
+  int i;
+  for (i = 0; i < wizard.page_num; i++) {
+    wizard.page[i].wizard = &wizard;
+
+    if(wizard.page[i].setup)
+      wizard.page[i].widget = 
+	wizard.page[i].setup(&wizard.page[i]);
+    else {
+      char *str = g_strdup_printf("Page %d", i);
+      wizard.page[i].widget = gtk_label_new(str);
+      g_free(str);
+    }
+
+    page[i].index = gtk_assistant_append_page(GTK_ASSISTANT(wizard.assistant),
+					      wizard.page[i].widget);
+
+    gtk_assistant_set_page_title(GTK_ASSISTANT(wizard.assistant),
+                                  wizard.page[i].widget, wizard.page[i].title);
+    gtk_assistant_set_page_type(GTK_ASSISTANT(wizard.assistant),
+                                  wizard.page[i].widget, wizard.page[i].type);
+
+    /* Set the introduction and conclusion pages as complete so they can be
+     * incremented or closed. */
+    gtk_assistant_set_page_complete(GTK_ASSISTANT(wizard.assistant),
+                     wizard.page[i].widget, wizard.page[i].complete);
+
+    if(wizard.page[i].update)
+      wizard.page[i].update(&wizard.page[i]);
+  }
+
+  /* install handler for timed updates */
+  wizard.handler_id = gtk_timeout_add(1000, wizard_update, &wizard);
+
+  /* make it a modal subdialog of the main window */
+  gtk_window_set_modal(GTK_WINDOW(wizard.assistant), TRUE);
+  gtk_window_set_transient_for(GTK_WINDOW(wizard.assistant), 
+			       GTK_WINDOW(appdata->window));
+
+  gtk_widget_show_all(wizard.assistant);
+
+  g_signal_connect(G_OBJECT(wizard.assistant), "destroy",
+		   G_CALLBACK(on_assistant_destroy), &wizard);
+
+  g_signal_connect(G_OBJECT(wizard.assistant), "cancel",
+		   G_CALLBACK(on_assistant_cancel), &wizard);
+
+  g_signal_connect(G_OBJECT(wizard.assistant), "close",
+		   G_CALLBACK(on_assistant_close), &wizard);
+
+  do {
+    if(gtk_events_pending()) 
+      gtk_main_iteration();
+    else 
+      usleep(1000);
+
+  } while(wizard.running);
+
+  gtk_timeout_remove(wizard.handler_id);
+
+  gtk_widget_destroy(wizard.assistant);
+}
+
+
 // vim:et:ts=8:sw=2:sts=2:ai
