@@ -21,7 +21,7 @@
 #define RELATION_EDIT_H
 
 void relation_add_dialog(appdata_t *appdata, object_t *object);
-void relation_list(appdata_t *appdata);
+void relation_list(GtkWidget *parent, appdata_t *appdata, object_t *object);
 void relation_show_members(GtkWidget *parent, relation_t *relation);
 
 #endif // RELATION_EDIT_H
